/*
 * Generated by gdbus-codegen 2.61.2 from org.gnome.SessionManager.xml. DO NOT EDIT.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from.
 */

#ifndef __DBUS_INHIBIT_H__
#define __DBUS_INHIBIT_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SessionManager */

#define DBUS_TYPE_SESSION_MANAGER (dbus_session_manager_get_type ())
#define DBUS_SESSION_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_SESSION_MANAGER, DBusSessionManager))
#define DBUS_IS_SESSION_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_SESSION_MANAGER))
#define DBUS_SESSION_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), DBUS_TYPE_SESSION_MANAGER, DBusSessionManagerIface))

struct _DBusSessionManager;
typedef struct _DBusSessionManager DBusSessionManager;
typedef struct _DBusSessionManagerIface DBusSessionManagerIface;

struct _DBusSessionManagerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_inhibit) (
    DBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags);

  gboolean (*handle_uninhibit) (
    DBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    guint arg_inhibit_cookie);

};

GType dbus_session_manager_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *dbus_session_manager_interface_info (void);
guint dbus_session_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void dbus_session_manager_complete_inhibit (
    DBusSessionManager *object,
    GDBusMethodInvocation *invocation,
    guint inhibit_cookie);

void dbus_session_manager_complete_uninhibit (
    DBusSessionManager *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void dbus_session_manager_call_inhibit (
    DBusSessionManager *proxy,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_session_manager_call_inhibit_finish (
    DBusSessionManager *proxy,
    guint *out_inhibit_cookie,
    GAsyncResult *res,
    GError **error);

gboolean dbus_session_manager_call_inhibit_sync (
    DBusSessionManager *proxy,
    const gchar *arg_app_id,
    guint arg_toplevel_xid,
    const gchar *arg_reason,
    guint arg_flags,
    guint *out_inhibit_cookie,
    GCancellable *cancellable,
    GError **error);

void dbus_session_manager_call_uninhibit (
    DBusSessionManager *proxy,
    guint arg_inhibit_cookie,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean dbus_session_manager_call_uninhibit_finish (
    DBusSessionManager *proxy,
    GAsyncResult *res,
    GError **error);

gboolean dbus_session_manager_call_uninhibit_sync (
    DBusSessionManager *proxy,
    guint arg_inhibit_cookie,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define DBUS_TYPE_SESSION_MANAGER_PROXY (dbus_session_manager_proxy_get_type ())
#define DBUS_SESSION_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_SESSION_MANAGER_PROXY, DBusSessionManagerProxy))
#define DBUS_SESSION_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_SESSION_MANAGER_PROXY, DBusSessionManagerProxyClass))
#define DBUS_SESSION_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_SESSION_MANAGER_PROXY, DBusSessionManagerProxyClass))
#define DBUS_IS_SESSION_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_SESSION_MANAGER_PROXY))
#define DBUS_IS_SESSION_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_SESSION_MANAGER_PROXY))

typedef struct _DBusSessionManagerProxy DBusSessionManagerProxy;
typedef struct _DBusSessionManagerProxyClass DBusSessionManagerProxyClass;
typedef struct _DBusSessionManagerProxyPrivate DBusSessionManagerProxyPrivate;

struct _DBusSessionManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  DBusSessionManagerProxyPrivate *priv;
};

struct _DBusSessionManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType dbus_session_manager_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DBusSessionManagerProxy, g_object_unref)
#endif

void dbus_session_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
DBusSessionManager *dbus_session_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
DBusSessionManager *dbus_session_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void dbus_session_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
DBusSessionManager *dbus_session_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
DBusSessionManager *dbus_session_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define DBUS_TYPE_SESSION_MANAGER_SKELETON (dbus_session_manager_skeleton_get_type ())
#define DBUS_SESSION_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), DBUS_TYPE_SESSION_MANAGER_SKELETON, DBusSessionManagerSkeleton))
#define DBUS_SESSION_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), DBUS_TYPE_SESSION_MANAGER_SKELETON, DBusSessionManagerSkeletonClass))
#define DBUS_SESSION_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), DBUS_TYPE_SESSION_MANAGER_SKELETON, DBusSessionManagerSkeletonClass))
#define DBUS_IS_SESSION_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), DBUS_TYPE_SESSION_MANAGER_SKELETON))
#define DBUS_IS_SESSION_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), DBUS_TYPE_SESSION_MANAGER_SKELETON))

typedef struct _DBusSessionManagerSkeleton DBusSessionManagerSkeleton;
typedef struct _DBusSessionManagerSkeletonClass DBusSessionManagerSkeletonClass;
typedef struct _DBusSessionManagerSkeletonPrivate DBusSessionManagerSkeletonPrivate;

struct _DBusSessionManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  DBusSessionManagerSkeletonPrivate *priv;
};

struct _DBusSessionManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType dbus_session_manager_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (DBusSessionManagerSkeleton, g_object_unref)
#endif

DBusSessionManager *dbus_session_manager_skeleton_new (void);


G_END_DECLS

#endif /* __DBUS_INHIBIT_H__ */
